/*
 * ZBOSS Zigbee software protocol stack
 *
 * Copyright (c) 2012-2020 DSR Corporation, Denver CO, USA.
 * http://www.dsr-zboss.com
 * http://www.dsr-corporation.com
 * All rights reserved.
 *
 *
 * Use in source and binary forms, redistribution in binary form only, with
 * or without modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 2. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 3. This software must only be used in or with a processor manufactured by Nordic
 *    Semiconductor ASA, or in or with a processor manufactured by a third party that
 *    is used in combination with a processor manufactured by Nordic Semiconductor.
 *
 * 4. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* PURPOSE: Error indication implementation
*/


#define ZB_TRACE_FILE_ID 116
#include "zboss_api_core.h"
#include "zb_error_indication.h"

#ifdef ZB_TOOL
zb_error_indication_global_t s_err_ind;
#define ERR_IND s_err_ind
#else

#include "zb_common.h"

#define ERR_IND ZG->err_ind
#endif

void zb_register_critical_error_handler(zb_error_handler_cb_t cb)
{
  TRACE_MSG(TRACE_COMMON1, ">> zb_register_critical_error_handler, cb %p",
            (FMT__P, cb));

#ifdef ZB_USE_ERROR_INDICATION
  if (cb)
  {
    ERR_IND.critical_err_handler_cb = cb;
  }
#else
  ZVUNUSED(cb);
#endif
  TRACE_MSG(TRACE_COMMON1, "<< zb_register_critical_error_handler", (FMT__0));
}

void zb_register_non_critical_error_handler(zb_error_handler_cb_t cb)
{
  TRACE_MSG(TRACE_COMMON1, ">> zb_register_non_critical_error_handler, cb %p",
            (FMT__P, cb));

#ifdef ZB_USE_ERROR_INDICATION
  if (cb)
  {
    ERR_IND.non_critical_err_handler_cb = cb;
  }
#else
  ZVUNUSED(cb);
#endif

  TRACE_MSG(TRACE_COMMON1, "<< zb_register_non_critical_error_handler", (FMT__0));
}
void zb_critical_error_indication(zb_uint8_t error_code)
{
  TRACE_MSG(TRACE_ERROR, ">> zb_critical_error_indication err_code %i",
            (FMT__H, error_code));

#ifdef ZB_USE_ERROR_INDICATION
  if (ERR_IND.critical_err_handler_cb)
  {
    ERR_IND.critical_err_handler_cb(error_code);
  }
  else
  {
    TRACE_MSG(TRACE_ERROR, "Critical error: code %i", (FMT__H, error_code));
#ifdef ZB_ZCL_SUPPORT_CLUSTER_DIAGNOSTICS
    ZB_ZCL_DIAGNOSTICS_INC(ZB_ZCL_ATTR_DIAGNOSTICS_NUMBER_OF_RESETS_ID);
#ifdef ZB_USE_NVRAM
    zb_nvram_write_dataset(ZB_NVRAM_HA_DATA);
#endif
#endif
    /* Call assert to send NWK report if possible, call reset if asserts are disabled by some reason. */
#ifdef USE_ASSERT
    ZB_ASSERT(0);
#else
    zb_reset(0);
#endif
  }
#else
  ZVUNUSED(error_code);
#endif

  TRACE_MSG(TRACE_ERROR, "<< zb_critical_error_indication", (FMT__0));
}

void zb_non_critical_error_indication(zb_uint8_t error_code)
{
  TRACE_MSG(TRACE_COMMON1, ">> zb_non_critical_error_indication err_code %i",
            (FMT__H, error_code));

#ifdef ZB_USE_ERROR_INDICATION
  if (ERR_IND.non_critical_err_handler_cb)
  {
    ERR_IND.non_critical_err_handler_cb(error_code);
  }
  else
  {
    TRACE_MSG(TRACE_ERROR, "Non-critical error: code %i", (FMT__H, error_code));
  }
#else
  ZVUNUSED(error_code);
#endif

  TRACE_MSG(TRACE_COMMON1, "<< zb_non_critical_error_indication", (FMT__0));
}

