/*
 * ZBOSS Zigbee software protocol stack
 *
 * Copyright (c) 2012-2020 DSR Corporation, Denver CO, USA.
 * http://www.dsr-zboss.com
 * http://www.dsr-corporation.com
 * All rights reserved.
 *
 *
 * Use in source and binary forms, redistribution in binary form only, with
 * or without modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 2. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 3. This software must only be used in or with a processor manufactured by Nordic
 *    Semiconductor ASA, or in or with a processor manufactured by a third party that
 *    is used in combination with a processor manufactured by Nordic Semiconductor.
 *
 * 4. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* PURPOSE: Internal BDB header file.
*/

#ifndef ZB_BDB_COMISSIONING_H
#define ZB_BDB_COMISSIONING_H 1

/**
 *  \addtogroup bdb_mode_commissioning
 *  @{
 *    @details
 *
 */

#define ZB_BDB_COMM_QUERY_TIME        4

typedef enum bdb_join_machine_e
{
  ZB_BDB_JOIN_MACHINE_SECONDARY_SCAN_START = 0,
  ZB_BDB_JOIN_MACHINE_PRIMARY_SCAN = 1,
  ZB_BDB_JOIN_MACHINE_SECONDARY_SCAN_DONE = 3,
  ZB_BDB_JOIN_MACHINE_DEVINFO_GATHER = 4,
  ZB_BDB_JOIN_MACHINE_ADDING_TO_NETWORK = 5,
} bdb_join_machine_t;


/** @cond touchlink */
#define ZB_BDB_TOUCHLINK_COMMISSIONING_IS_IN_PROGRESS()                                                                                     \
  (((ZB_BDB().bdb_commissioning_mode == ZB_BDB_TOUCHLINK_TARGET) || (ZB_BDB().bdb_commissioning_mode == ZB_BDB_TOUCHLINK_COMMISSIONING)) && \
   (ZB_BDB().bdb_commissioning_status == ZB_BDB_STATUS_IN_PROGRESS))
/** @endcond */ /* touchlink */

void bdb_init(void);

void bdb_commissioning_machine(zb_uint8_t param);

void zb_bdb_finding_binding_init_ctx(void);

void zb_bdb_process_identify_query_res(zb_uint8_t param);

/** @cond touchlink */
void bdb_touchlink_target_start(zb_uint8_t param);

void bdb_touchlink_initiator(zb_uint8_t param);
/** @endcond */ /* touchlink */

#ifdef ZB_ROUTER_ROLE
void bdb_remove_joiner(zb_uint8_t param);
#endif

void bdb_check_fn(void);

void bdb_initiate_commissioning(zb_uint8_t param);

void bdb_preinit(void);

void bdb_commissioning_leave_done(void);

void bdb_start_rejoin_recovery(zb_uint8_t param, zb_uint16_t user_param);

zb_bool_t bdb_joined(void);

/** @}  */ /* bdb_mode_comissioning */

/** @} */ /* bdb_mode_commissioning */

#endif /* ZB_BDB_COMISSIONING_H */
