/*
 * ZBOSS Zigbee software protocol stack
 *
 * Copyright (c) 2012-2020 DSR Corporation, Denver CO, USA.
 * http://www.dsr-zboss.com
 * http://www.dsr-corporation.com
 * All rights reserved.
 *
 *
 * Use in source and binary forms, redistribution in binary form only, with
 * or without modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 2. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 3. This software must only be used in or with a processor manufactured by Nordic
 *    Semiconductor ASA, or in or with a processor manufactured by a third party that
 *    is used in combination with a processor manufactured by Nordic Semiconductor.
 *
 * 4. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* PURPOSE: Error indication globals definition
*/

#ifndef ZB_ERROR_INDICATION_H
#define ZB_ERROR_INDICATION_H 1

#include "zb_types.h"

typedef enum zb_critical_error_code_e
{
  /* common MCU subsystem */
  ZB_CRITICAL_ERROR_HARDFAULT = 11,
  /* Alien MAC subsystem */
  ZB_CRITICAL_ERROR_ALIEN_MAC_ASSERT = 21,
  /* ZBOSS subsystem */
  ZB_CRITICAL_ERROR_ZBOSS_ASSERT = 31,
  ZB_CRITICAL_ERROR_ZBOSS_ZDO_START,
  ZB_CRITICAL_ERROR_ZBOSS_ZDO_STARTUP_COMPLETE,
  ZB_CRITICAL_ERROR_WATCHDOG,
  ZB_CRITICAL_ERROR_MAC_RESET,
  /* LWIP subsystem */
  ZB_CRITICAL_ERROR_LWIP_NO_ETH_LINK = 41,
  ZB_CRITICAL_ERROR_LWIP_ETH_LINK_DOWN,
  ZB_CRITICAL_ERROR_LWIP_NO_IP_ADDR,
  ZB_CRITICAL_ERROR_LWIP_ASSERT,
  ZB_CRITICAL_ERROR_LWIP_DHCP_TIMEOUT,
  /* Cloud subsystem */
  ZB_CRITICAL_ERROR_CLOUD_CANT_GET_CMD = 51,
  /* NVRAM */
  ZB_CRITICAL_ERROR_NVRAM_READ_ERROR = 71,
  ZB_CRITICAL_ERROR_NVRAM_PRECOMMISSIONING_ERROR = 72,
  /* TODO: Add error codes for NVRAM and GreenPower subsystems */
} zb_critical_error_code_t;

typedef void (*zb_error_handler_cb_t)(zb_uint8_t error_code);

typedef struct zb_error_indication_global_s
{
  zb_error_handler_cb_t critical_err_handler_cb;
  zb_error_handler_cb_t non_critical_err_handler_cb;
  zb_error_handler_cb_t saved_err_handler_cb;
} zb_error_indication_global_t;

void zb_register_critical_error_handler(zb_error_handler_cb_t cb);
void zb_register_non_critical_error_handler(zb_error_handler_cb_t cb);
void zb_critical_error_indication(zb_uint8_t error_code);
void zb_non_critical_error_indication(zb_uint8_t error_code);

#ifdef ZB_NO_CRITICAL_ERRORS_INDICATION
  #define ZB_CRITICAL_ERROR_INDICATION(err_code)
  #define ZB_NON_CRITICAL_ERROR_INDICATION(err_code)
#else
  #define ZB_CRITICAL_ERROR_INDICATION(err_code) zb_critical_error_indication((err_code))
  #define ZB_NON_CRITICAL_ERROR_INDICATION(err_code) zb_non_critical_error_indication((err_code))
#endif

#endif  /* ZB_ERROR_INDICATION_H */
