/*
 * ZBOSS Zigbee software protocol stack
 *
 * Copyright (c) 2012-2020 DSR Corporation, Denver CO, USA.
 * http://www.dsr-zboss.com
 * http://www.dsr-corporation.com
 * All rights reserved.
 *
 *
 * Use in source and binary forms, redistribution in binary form only, with
 * or without modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 2. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 3. This software must only be used in or with a processor manufactured by Nordic
 *    Semiconductor ASA, or in or with a processor manufactured by a third party that
 *    is used in combination with a processor manufactured by Nordic Semiconductor.
 *
 * 4. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* PURPOSE: Some macros for fast calculation

*/
#ifndef MAGIC_MACROSES_H
#define MAGIC_MACROSES_H 1

/*! @cond internals_doc */
/**
   @addtogroup ZB_BASE
   @{
*/

#define MAGIC_ROUND_A_TO_B(a, b) (((a) + ((b)-1)) & -((zb_int_t)(b)))

#define MAGIC_ROUND_TO_32(a) (((a) + ((32)-1)) & (zb_uint8_t)(-(32)))

#if 0//ndef ZB_IAR
/* Causing warnings about a sign change. Let's do ont use it. */
#define MAGIC_ROUND_TO_4(a) (((a) + ((4)-1)) & -(4))
#else
#define MAGIC_ROUND_TO_4(a) (((a) + 3) / 4 * 4)
#endif

#define MAGIC_ROUND_TO_INT_SIZE(a) (((a) + (sizeof(zb_int_t)-1)) & -((zb_int_t)sizeof(zb_int_t)))

#define MAGIC_TRUNC_TO_4(a) ((a) & (~3))

#define MAGIC_TRUNC_TO_8(a) ((a) & (~7))

#define MAGIC_LEAST_SIGNIFICANT_BIT_MASK(x) ((x) & (-((zb_int_t)(x))))

#define MAGIC_LS_ZERO_MASK(x) ((~(x)) & ((x) + 1))

#define MAGIC_TURN_OFF_LS_1(x) ((x) & ((x) - 1))

/**
 * Turn off (set to 0) rightmost chain of continuous 1s
 */
#define MAGIC_TURN_OFF_RIGHT_ONES(x)  ((((x) | ((x) - 1)) + 1) & (x))


#define MAGIC_ISOLATE_RIGHT_ONES(x)  ((x) ^ MAGIC_TURN_OFF_RIGHT_ONES(x))

/**
 * Branch-free calculation of x = (x == a ? b : a)
 */
#define MAGIC_ALTERNATE(x, a, b) ((a) ^ (b) ^ (x))


#define MAGIC_AVG(a, b) ((a) & (b) + ((a) ^ (b))/2)

/**
   return -1, 0, 1 if x < y, x == y, x > y
  */
#define MAGIC_CMP(x, y) (((x) > (y)) - ((x) < (y)))

/**
   Return signum of the result: -1 if < 0, 0 if == 0, else 1
 */
#define MAGIC_SIGN(x) (((x) > 0) - ((x) < 0))

/**
   Return 1 if the number is a power of 2, works only if x > 0
 */
#define MAGIC_IS_POWER_OF_TWO(x) ( ((x) & ((x) - 1)) == 0 )


/*
Following set of macros implements compile-time calculation of # of bits in the
word using following code:

 32-bit recursive reduction using SWAR...
     but first step is mapping 2-bit values
     into sum of 2 1-bit values in sneaky way
  x -= ((x >> 1) & 0x55555555);
  x = (((x >> 2) & 0x33333333) + (x & 0x33333333));
  x = (((x >> 4) + x) & 0x0f0f0f0f);
  x += (x >> 8);
  x += (x >> 16);
  return(x & 0x0000003f);
*/


#define MAGIC_ONE_1(x) ((x) - (((x) >> 1) & 0x55555555))
#define MAGIC_ONE_2(x) (((MAGIC_ONE_1(x) >> 2) & 0x33333333) + (MAGIC_ONE_1(x) & 0x33333333))
#define MAGIC_ONE_4(x) (((MAGIC_ONE_2(x) >> 4) + MAGIC_ONE_2(x)) & 0x0f0f0f0f)
#define MAGIC_ONE_8(x) (MAGIC_ONE_4(x) + (MAGIC_ONE_4(x) >> 8))
#define MAGIC_ONE_16(x) (MAGIC_ONE_8(x) + (MAGIC_ONE_8(x) >> 16))
#define MAGIC_ONE_32(x) (MAGIC_ONE_16(x) & 0x0000003f)


#define MAGIC_LOG2_1(x) ((x) | ((x) >> 1))
#define MAGIC_LOG2_2(x) (MAGIC_LOG2_1(x) | (MAGIC_LOG2_1(x) >> 2))
#define MAGIC_LOG2_4(x) (MAGIC_LOG2_2(x) | (MAGIC_LOG2_2(x) >> 4))
#define MAGIC_LOG2_8(x) (MAGIC_LOG2_4(x) | (MAGIC_LOG2_4(x) >> 8))
#define MAGIC_LOG2_16(x) (MAGIC_LOG2_8(x) | (MAGIC_LOG2_8(x) >> 16))

#define MAGIC_LOG2_32(x) (MAGIC_ONE_32(MAGIC_LOG2_16(x)) - 1)

/*! @} */
/*! @endcond */

#endif /* MAGIC_MACROSES_H */
