/*
 * ZBOSS Zigbee software protocol stack
 *
 * Copyright (c) 2012-2020 DSR Corporation, Denver CO, USA.
 * http://www.dsr-zboss.com
 * http://www.dsr-corporation.com
 * All rights reserved.
 *
 *
 * Use in source and binary forms, redistribution in binary form only, with
 * or without modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 2. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 3. This software must only be used in or with a processor manufactured by Nordic
 *    Semiconductor ASA, or in or with a processor manufactured by a third party that
 *    is used in combination with a processor manufactured by Nordic Semiconductor.
 *
 * 4. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*  PURPOSE: NCP High level transport (adapters layer) implementation for the host side: APS category
*/
#define ZB_TRACE_FILE_ID 17513

#include "ncp_host_hl_transport_internal_api.h"
#include "ncp_host_soc_state.h"

void ncp_host_handle_secur_tclk_indication(zb_ret_t status,
                                           zb_ieee_addr_t trust_center_address,
                                           zb_uint8_t key_type)
{
  TRACE_MSG(TRACE_TRANSPORT2, ">> ncp_host_handle_secur_tclk_indication, status 0x%x",
            (FMT__D, status));

  TRACE_MSG(TRACE_TRANSPORT2, "status 0x%x", (FMT__D, status));

  ncp_host_state_set_waiting_for_tclk(ZB_FALSE);

  if (status == RET_OK)
  {
    TRACE_MSG(TRACE_TRANSPORT2, "tc address" TRACE_FORMAT_64 ", key_type 0x%x",
              (FMT__A_D, TRACE_ARG_64(trust_center_address), key_type));

    zb_aib_set_trust_center_address(trust_center_address);

    /* TODO: handle key_type properly, when we use more than one type */
    ZB_ASSERT(key_type == 0);

    ncp_host_state_set_tclk_valid(ZB_TRUE);

    zb_buf_get_out_delayed(zdo_commissioning_tclk_update_complete);
  }
  else
  {
    zb_buf_get_out_delayed(zdo_commissioning_tclk_update_failed);
  }


  TRACE_MSG(TRACE_TRANSPORT2, "<< ncp_host_handle_secur_tclk_indication", (FMT__0));
}
