/*
 * ZBOSS Zigbee software protocol stack
 *
 * Copyright (c) 2012-2020 DSR Corporation, Denver CO, USA.
 * http://www.dsr-zboss.com
 * http://www.dsr-corporation.com
 * All rights reserved.
 *
 *
 * Use in source and binary forms, redistribution in binary form only, with
 * or without modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 2. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 3. This software must only be used in or with a processor manufactured by Nordic
 *    Semiconductor ASA, or in or with a processor manufactured by a third party that
 *    is used in combination with a processor manufactured by Nordic Semiconductor.
 *
 * 4. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*  PURPOSE: NCP High level transport implementation for the host side.
 *  API of the NCP High level transport, which is not public and used only by adapter layer */

#ifndef NCP_HOST_HL_TRANSPORT_INTERNAL_API_H
#define NCP_HOST_HL_TRANSPORT_INTERNAL_API_H 1

#include "zb_common.h"
#include "zb_types.h"

#include "zb_aps.h"
#include "ncp/ncp_host_api.h"

typedef ZB_PACKED_PRE struct ncp_proto_network_descriptor_s
{
  zb_ieee_addr_t extended_pan_id;
  zb_uint16_t    pan_id;
  zb_uint8_t     nwk_update_id;
  zb_uint8_t     channel_page;
  zb_uint8_t     channel;
  zb_uint8_t     flags;
} ZB_PACKED_STRUCT ncp_proto_network_descriptor_t;

typedef ZB_PACKED_PRE struct ncp_host_zb_apsde_data_req_s
{
  zb_addr_u   dst_addr;
  zb_uint16_t profileid;
  zb_uint16_t clusterid;
  zb_uint8_t  dst_endpoint;
  zb_uint8_t  src_endpoint;
  zb_uint8_t  radius;
  zb_uint8_t  addr_mode;
  zb_uint8_t  tx_options;
  zb_uint8_t  use_alias;
  zb_uint16_t alias_src_addr;
  zb_uint8_t  alias_seq_num;
} ZB_PACKED_STRUCT ncp_host_zb_apsde_data_req_t;

/* NCP High level transport functions, called from Adapters layer */
zb_ret_t ncp_host_apsde_data_request(ncp_host_zb_apsde_data_req_t *req, zb_uint8_t param_len,
                                     zb_uint16_t data_len, zb_uint8_t *data_ptr,
                                     zb_uint8_t *tsn);

zb_ret_t ncp_host_apsme_bind_unbind_request(zb_ieee_addr_t src_addr, zb_uint8_t src_endpoint,
                                            zb_uint16_t cluster_id, zb_uint8_t addr_mode,
                                            zb_addr_u dst_addr, zb_uint8_t dst_endpoint,
                                            zb_bool_t is_bind, zb_uint8_t *tsn);

zb_ret_t ncp_host_apsme_add_group_request(zb_apsme_add_group_req_t *req, zb_uint8_t *tsn);

zb_ret_t ncp_host_apsme_remove_group_request(zb_apsme_remove_group_req_t *req, zb_uint8_t *tsn);

zb_ret_t ncp_host_apsme_remove_all_groups_request(zb_apsme_remove_all_groups_req_t *req, zb_uint8_t *tsn);

zb_ret_t ncp_host_aps_check_binding_request(zb_aps_check_binding_req_t *req, zb_uint8_t *tsn);

zb_ret_t ncp_host_aps_get_group_table_request(void);

zb_ret_t ncp_host_nwk_formation(zb_uint32_t *scan_channels_list, zb_uint8_t scan_duration,
                                zb_uint8_t distributed_network, zb_uint16_t distributed_network_address);

zb_ret_t ncp_host_nwk_start_without_formation(void);

zb_ret_t ncp_host_nwk_discovery(zb_uint32_t *scan_channels_list, zb_uint8_t scan_duration);

zb_ret_t ncp_host_nwk_nlme_join(zb_ext_pan_id_t extended_pan_id, zb_uint8_t rejoin_network,
                                zb_uint32_t *scan_channels_list, zb_uint8_t scan_duration,
                                zb_uint8_t mac_capabilities, zb_uint8_t security_enable);

zb_ret_t ncp_host_nwk_permit_joining(zb_uint8_t permit_duration);

zb_ret_t ncp_host_zdo_ieee_addr_request(zb_uint8_t *ncp_tsn, zb_zdo_ieee_addr_req_param_t *req_param);

zb_ret_t ncp_host_zdo_nwk_addr_request(zb_uint8_t *ncp_tsn, zb_zdo_nwk_addr_req_param_t *req_param);

zb_ret_t ncp_host_zdo_power_desc_request(zb_uint8_t *ncp_tsn, zb_uint16_t nwk_addr);

zb_ret_t ncp_host_zdo_node_desc_request(zb_uint8_t *ncp_tsn, zb_uint16_t nwk_addr);

zb_ret_t ncp_host_zdo_simple_desc_request(zb_uint8_t *ncp_tsn, zb_uint16_t nwk_addr, zb_uint8_t endpoint);

zb_ret_t ncp_host_zdo_active_ep_request(zb_uint8_t *ncp_tsn, zb_uint16_t nwk_addr);

zb_ret_t ncp_host_zdo_match_desc_request(zb_uint8_t *ncp_tsn, zb_zdo_match_desc_param_t *req);

zb_ret_t ncp_host_zdo_bind_unbind_request(zb_uint8_t *ncp_tsn, zb_zdo_bind_req_param_t *bind_param,
                                          zb_bool_t is_bind);

zb_ret_t ncp_host_zdo_permit_joining_request(zb_uint8_t *ncp_tsn, zb_uint16_t short_addr,
                                             zb_uint8_t permit_duration, zb_uint8_t tc_significance);

zb_ret_t ncp_host_zdo_mgmt_leave_request(zb_uint8_t *ncp_tsn, zb_zdo_mgmt_leave_param_t *req);

zb_ret_t ncp_host_zdo_rejoin_request(zb_uint8_t *ncp_tsn,
                                     zb_ext_pan_id_t ext_pan_id,
                                     zb_channel_page_t *channels_list,
                                     zb_uint8_t secure_rejoin);

zb_ret_t ncp_host_zdo_system_server_discovery_request(zb_uint8_t *ncp_tsn, zb_uint16_t server_mask);

zb_ret_t ncp_host_nwk_nlme_start_router_request(zb_uint8_t beacon_order, zb_uint8_t superframe_order,
                                                zb_uint8_t battery_life_extension);

/* Functions to be called from NCP High level transport */
void ncp_host_handle_apsde_data_indication(zb_apsde_data_indication_t *ind,
                                           zb_uint8_t *data_ptr,
                                           zb_uint8_t params_len, zb_uint16_t data_len);

void ncp_host_handle_apsde_data_response(zb_apsde_data_confirm_t *conf, zb_uint8_t tsn);

void ncp_host_handle_apsme_bind_response(zb_ret_t status, zb_uint8_t tsn);

void ncp_host_handle_apsme_unbind_response(zb_ret_t status, zb_uint8_t tsn);

void ncp_host_handle_apsme_add_group_response(zb_ret_t status, zb_uint8_t tsn);
void ncp_host_handle_apsme_remove_group_response(zb_ret_t status, zb_uint8_t tsn);
void ncp_host_handle_apsme_remove_all_groups_response(zb_ret_t status, zb_uint8_t tsn);
void ncp_host_handle_aps_check_binding_response(zb_ret_t status, zb_bool_t exists, zb_uint8_t tsn);
void ncp_host_handle_aps_get_group_table_response(zb_ret_t status_code,
                                                  zb_aps_group_table_ent_t* table,
                                                  zb_size_t entry_cnt);

void ncp_host_handle_nwk_formation_response(zb_ret_t status,
                                            zb_uint16_t short_address,
                                            zb_uint16_t pan_id,
                                            zb_uint8_t current_page,
                                            zb_uint8_t current_channel);

void ncp_host_handle_nwk_start_without_formation_response(zb_ret_t status);

void ncp_host_handle_nwk_discovery_response(zb_ret_t status, zb_uint8_t network_count,
                                            ncp_proto_network_descriptor_t *network_descriptors);

void ncp_host_handle_nwk_permit_joining_response(zb_ret_t status);

void ncp_host_handle_nwk_nlme_join_response(zb_ret_t status, zb_uint16_t short_addr, zb_ext_pan_id_t extended_pan_id,
                                            zb_uint8_t channel_page, zb_uint8_t logical_channel,
                                            zb_uint8_t enhanced_beacon, zb_uint8_t mac_interface);

void ncp_host_handle_nwk_joined_indication(zb_uint16_t nwk_addr, zb_ext_pan_id_t ext_pan_id,
                                           zb_uint8_t channel_page, zb_uint8_t channel,
                                           zb_uint8_t beacon_type, zb_uint8_t mac_interface);

void ncp_host_handle_nwk_join_failed_indication(zb_uint8_t status_category, zb_uint8_t status_code);

void ncp_host_handle_nwk_address_update_indication(zb_uint16_t nwk_addr);
#ifdef ZB_APSDE_REQ_ROUTING_FEATURES
void ncp_host_handle_nwk_route_reply_indication(zb_uint16_t nwk_addr);

void ncp_host_handle_nwk_route_request_send_indication(zb_uint16_t nwk_addr);

void ncp_host_handle_nwk_route_record_send_indication(zb_bufid_t buf);
#endif
void ncp_host_handle_zdo_ieee_addr_response(zb_ret_t status, zb_uint8_t ncp_tsn,
                                            zb_zdo_ieee_addr_resp_t *resp,
                                            zb_bool_t is_extended,
                                            zb_zdo_ieee_addr_resp_ext_t *resp_ext,
                                            zb_zdo_ieee_addr_resp_ext2_t *resp_ext2,
                                            zb_uint16_t *addr_list);

void ncp_host_handle_zdo_nwk_addr_response(zb_ret_t status, zb_uint8_t ncp_tsn,
                                           zb_zdo_nwk_addr_resp_head_t *resp,
                                           zb_bool_t is_extended,
                                           zb_zdo_nwk_addr_resp_ext_t *resp_ext,
                                           zb_zdo_nwk_addr_resp_ext2_t *resp_ext2,
                                           zb_uint16_t *addr_list);

void ncp_host_handle_zdo_power_descriptor_response(zb_ret_t status, zb_uint8_t ncp_tsn, zb_uint16_t nwk_addr,
                                                   zb_uint16_t power_descriptor);

void ncp_host_handle_zdo_node_descriptor_response(zb_ret_t status, zb_uint8_t ncp_tsn, zb_uint16_t nwk_addr,
                                                  zb_af_node_desc_t *node_desc);

void ncp_host_handle_zdo_simple_descriptor_response(zb_ret_t status, zb_uint8_t ncp_tsn, zb_uint16_t nwk_addr,
                                                    zb_af_simple_desc_1_1_t *simple_desc,
                                                    zb_uint16_t *app_cluster_list_ptr);

void ncp_host_handle_zdo_active_ep_response(zb_ret_t status, zb_uint8_t ncp_tsn, zb_uint16_t nwk_addr,
                                            zb_uint8_t ep_count, zb_uint8_t *ep_list);

void ncp_host_handle_zdo_match_desc_response(zb_ret_t status, zb_uint8_t ncp_tsn, zb_uint16_t nwk_addr,
                                             zb_uint8_t match_ep_count, zb_uint8_t *match_ep_list);

void ncp_host_handle_zdo_bind_response(zb_ret_t status, zb_uint8_t ncp_tsn);

void ncp_host_handle_zdo_unbind_response(zb_ret_t status, zb_uint8_t ncp_tsn);

void ncp_host_handle_zdo_permit_joining_response(zb_ret_t status, zb_uint8_t ncp_tsn);

void ncp_host_handle_zdo_mgmt_leave_response(zb_ret_t status, zb_uint8_t ncp_tsn);

void ncp_host_handle_zdo_rejoin_response(zb_ret_t status);

void ncp_host_handle_zdo_system_server_discovery_response(zb_ret_t status, zb_uint8_t ncp_tsn,
                                                          zb_uint16_t server_mask);

void ncp_host_handle_nwk_nlme_router_start_response(zb_ret_t status);

void ncp_host_handle_zdo_device_annce_indication(zb_uint16_t nwk_addr, zb_ieee_addr_t ieee_addr,
                                                 zb_uint8_t capability);


void ncp_host_handle_secur_tclk_indication(zb_ret_t status,
                                           zb_ieee_addr_t trust_center_address,
                                           zb_uint8_t key_type);



/**
 * Force NCP module reboot
 *
 */
zb_ret_t ncp_host_reset(void);

/**
 * Force NCP module reboot.
 * Cleans NCP module internal state and removes frmo NVRAM all info except NWK packets counter.
 */
zb_ret_t ncp_host_factory_reset(void);

/**
 * Force NCP module reboot.
 * Cleans NCP module internal state, erases all NVRAM content including NWK packet counter.
 *
 * @warning Use for debugging purposes only
 */
zb_ret_t ncp_host_reset_nvram_erase(void);

/**
 * Sends Get Module Version request.
 *
 */
zb_ret_t ncp_host_get_module_version(void);

/**
 * Sends Get Join Status request.
 *
 */
zb_ret_t ncp_host_get_join_status(void);

/**
 * Sends Get Authentication Status request.
 *
 */
zb_ret_t ncp_host_get_authentication_status(void);

/**
 * Requests device local IEEE address
 *
 * @param interface_num - Number of MAC interface. Always 0 for 2.4GHz-only build
 *
 */
zb_ret_t ncp_host_get_local_ieee_addr(zb_uint8_t interface_num);

/**
 * Sets device local IEEE address
 *
 * @note In the current version it works only in the ZBOSS build without ZB_PRODUCTION_CONFIG
 *
 * @param interface_num - Number of MAC interface. Always 0 for 2.4GHz-only build
 * @param addr - IEEE addresss that is set on the device
 */
zb_ret_t ncp_host_set_local_ieee_addr(zb_uint8_t interface_num, const zb_ieee_addr_t addr);

/**
 * Requests RxOnWhenIdle PIB attribute
 */
zb_ret_t ncp_host_get_rx_on_when_idle(void);


/**
 * Sets RxOnWhenIdle PIB attribute
 *
 * @param rx_on_when_idle - RxOnWhenIdle PIB attribute value
 */
zb_ret_t ncp_host_set_rx_on_when_idle(zb_bool_t rx_on_when_idle);


/**
 * Requests Keepalive Timeout
 */
zb_ret_t ncp_host_get_keepalive_timeout(void);


/**
 * Sets Keepalive Timeout value
 *
 * @param keepalive_timeout - Keepalive Timeout value
 */
zb_ret_t ncp_host_set_keepalive_timeout(zb_uint32_t keepalive_timeout);


/**
 * Requests End device Timeout
 */
zb_ret_t ncp_host_get_end_device_timeout(void);


/**
 * Sets End Device Timeout value
 *
 * @param timeout - End Device Timeout value
 */
zb_ret_t ncp_host_set_end_device_timeout(zb_uint8_t timeout);


/**
 * Sets keepalive mode
 *
 * @param mode - keepalive mode value
 */
zb_ret_t ncp_host_nwk_set_keepalive_mode(zb_uint8_t mode);


/**
 * Requests long poll interval value
 */
zb_ret_t ncp_host_nwk_get_long_poll_interval(zb_uint8_t *ncp_tsn);


/**
 * Requests "in fast poll" flag value
 */
zb_ret_t ncp_host_nwk_get_in_fast_poll_flag(zb_uint8_t *ncp_tsn);


/**
 * Sets fast poll timeout value
 *
 * @param timeout - timeout value
 */
zb_ret_t ncp_host_nwk_set_fast_poll_timeout(zb_time_t timeout);


/**
 * Permits turbo poll
 *
 * @param permit_status - permit status value
 */
zb_ret_t ncp_host_nwk_permit_turbo_poll(zb_bool_t permit_status);


/**
 * Sets Zigbee channel mask.
 *
 * @param page
 * @param channel_mask
 *
 */
zb_ret_t ncp_host_set_zigbee_channel_mask(zb_uint8_t page, zb_uint32_t channel_mask);

/**
 * Gets Zigbee channel mask.
 *
 */
zb_ret_t ncp_host_get_zigbee_channel_mask(void);


/**
 * Gets Current Zigbee Channel
 *
 */
zb_ret_t ncp_host_get_zigbee_channel(void);

/**
 * Sets Zigbee PAN ID
 *
 * @note It is not allowed to set Zigbee PAN ID if device is already joined to the network.
 *       In this case NCP returns status code INVALID_STATE.
 *
 * @param pan_id
 *
 */
zb_ret_t ncp_host_set_zigbee_pan_id(zb_uint16_t pan_id);

/**
 * Gets Zigbee PAN ID
 *
 */
zb_ret_t ncp_host_get_zigbee_pan_id(void);

/**
 * Sets Zigbee Role
 *
 * @note It is impossible to change NCP role after it is alreade commissioned.
 *
 * @param zigbee role
 *
 */
zb_ret_t ncp_host_set_zigbee_role(zb_uint8_t zigbee_role);

/**
 * Gets Zigbee Role
 *
 */
zb_ret_t ncp_host_get_zigbee_role(void);

/**
 *  Gets APS key by IEEE address
 *
 */
zb_ret_t ncp_host_get_aps_key_by_ieee(zb_ieee_addr_t ieee_addr);

/**
 * Sets NWK key
 *
 * @param key
 * @param key_number
 */
zb_ret_t ncp_host_set_nwk_key(zb_uint8_t *key, zb_uint8_t key_number);

/**
 * Requests NWK keys list
 *
 */
zb_ret_t ncp_host_get_nwk_keys(void);


/**
 * Gets current parent address
 */
zb_ret_t ncp_host_get_parent_address(void);

/**
 * Gets current extended PAN ID
 */
zb_ret_t ncp_host_get_extended_pan_id(void);

/**
 * Gets current coordinator version
 */
zb_ret_t ncp_host_get_coordinator_version(void);

/**
 * Gets current short address
 */
zb_ret_t ncp_host_get_short_address(void);

/**
 * Gets current trust center address
 */
zb_ret_t ncp_host_get_trust_center_address(void);

/**
 * Gets IEEE adress of the device by short address in the local address translation table
 * @ref zb_address_ieee_by_short()
 *
 * @param short_addr
 */
zb_ret_t ncp_host_nwk_get_ieee_by_short(zb_uint16_t short_addr);

/**
 * Gets entry from neighbor table by IEEE
 * @ref zb_nwk_neighbor_get_by_ieee()
 *
 * @param ieee_addr
 */
zb_ret_t ncp_host_nwk_get_neighbor_by_ieee(zb_ieee_addr_t ieee_addr);


/**
 * Sets Fast Poll Interval PIM attribute
 *
 * @param fast_poll_interval - Fast Poll Interval PIB attribute value
 */
zb_ret_t ncp_host_nwk_set_fast_poll_interval(zb_uint16_t fast_poll_interval);


/**
 * Sets Long Poll Interval PIM attribute
 *
 * @param long_poll_interval - Long Poll Interval PIB attribute value
 */
zb_ret_t ncp_host_nwk_set_long_poll_interval(zb_uint32_t long_poll_interval);


/**
 * Starts poll with the Fast Poll Interval specified by PIM attribute
 */
zb_ret_t ncp_host_nwk_start_fast_poll(void);


/**
 * Starts poll with the Long Poll Interval specified by PIM attribute
 */
zb_ret_t ncp_host_nwk_start_poll(void);


/**
 * Stops fast poll and restart it with the Long Poll Interval
 *
 * @param ncp_tsn
 */
zb_ret_t ncp_host_nwk_stop_fast_poll(zb_uint8_t *ncp_tsn);


/**
 * Stops poll
 */
zb_ret_t ncp_host_nwk_stop_poll(void);


/**
 * Enables turbo poll for given amount of time
 *
 * @param time - Turbo poll active time in milliseconds
 */
zb_ret_t ncp_host_nwk_enable_turbo_poll(zb_uint32_t time);


/**
 * Disables turbo poll
 */
zb_ret_t ncp_host_nwk_disable_turbo_poll(void);


/**
 * Gets short address from the local address trandlation table by IEEE
 * @ref zb_address_short_by_ieee()
 *
 * @param ieee_addr
 */
zb_ret_t ncp_host_nwk_get_short_by_ieee(zb_ieee_addr_t ieee_addr);


/**
 * Starts Turbo Poll for the specified packets count
 *
 * @param packets_count
 */
zb_ret_t ncp_host_nwk_start_turbo_poll_packets(zb_uint8_t packets_count);


/**
 * Starts Turbo Poll for the specified time interval in milliseconds
 *
 * @param turbo_poll_timeout_ms
 */
zb_ret_t ncp_host_nwk_start_turbo_poll_continuous(zb_time_t turbo_poll_timeout_ms);

/**
 * Leaves packets turbo poll process
 */
zb_ret_t ncp_host_nwk_turbo_poll_packets_leave(void);


/**
 * Leaves continuous turbo poll process
 */
zb_ret_t ncp_host_nwk_turbo_poll_continuous_leave(void);

/**
 * Adds the install code for the device with specified long address
 * @note This function is valid only for Trust Center (ZC)
 * @ref zb_secur_ic_add()
 *
 *  @param address - long address of the device to add the install code
 *  @param ic_type - install code type as enumerated in
 *  @param ic - pointer to the install code buffer
 *
 */
zb_ret_t ncp_host_secur_add_ic(zb_ieee_addr_t address, zb_uint8_t ic_type, zb_uint8_t *ic);



/* AF API */
/**
 * Sets Simple Descriptor for the device
 *
 * @param endpoint - Endpoint
 * @param app_profile_id - Application profile identifier
 * @param app_device_id - Application device identifier
 * @param app_device_version - Application device version
 * @param app_input_cluster_count - Application input cluster count
 * @param app_output_cluster_count - Application output cluster count
 * @param app_input_cluster_list - List of application input clusters
 * @param app_output_cluster_list - List of application output clusters
 */
zb_ret_t ncp_host_af_set_simple_descriptor(zb_uint8_t  endpoint, zb_uint16_t app_profile_id,
                                           zb_uint16_t app_device_id, zb_bitfield_t app_device_version,
                                           zb_uint8_t app_input_cluster_count,
                                           zb_uint8_t app_output_cluster_count,
                                           zb_uint16_t *app_input_cluter_list,
                                           zb_uint16_t *app_output_cluster_list);

/**
 * Deletes endpoint of the device
 *
 * @param endpoint - Number of the endpoint to delete
 */
zb_ret_t ncp_host_af_delete_endpoint(zb_uint8_t endpoint);

/**
 * Sets Node Descriptor
 *
 * @param device_type - Device type: ZC, ZR or ZED
 * @param mac_capabilities - MAC Capabilities bitfield
 * @param manufacturer_code - Manufacturer code
 */
zb_ret_t ncp_host_af_set_node_descriptor(zb_uint8_t device_type, zb_uint8_t mac_capabilities,
                                         zb_uint16_t manufacturer_code);

/**
 * Sets Power Descriptor
 * @ref zb_set_node_power_descriptor()
 *
 */
zb_ret_t ncp_host_af_set_power_descriptor(zb_uint8_t current_power_mode,
                                          zb_uint8_t available_power_sources,
                                          zb_uint8_t current_power_source,
                                          zb_uint8_t current_power_source_level);


/**
 * Handles reset response.
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_reset_response(zb_ret_t status_code, zb_bool_t is_solicited);

/**
 * Handles Get Version response.
 *
 * @param status_code - status of operation
 * @param version - received NCP module version
 *
 */
void ncp_host_handle_get_module_version_response(zb_ret_t status_code, zb_uint32_t version);

/**
 * Handles Get Join Status response
 *
 * @param status_code - status of operation
 * @param joined - 0 means devices is not joined, 1 means device is joined
 */
void ncp_host_handle_get_join_status_response(zb_ret_t status_code, zb_bool_t joined);

/**
 * Handles Get Authentication Status response
 *
 * @param status_code - status of operation
 * @param authenticated - 0 means devices is not authenticated, 1 means device is authenticated
 */
void ncp_host_handle_get_authentication_status_response(zb_ret_t status_code, zb_bool_t authenticated);

/**
 * Handles Get Local IEEE Address response.
 *
 * @param status_code - status of operation
 * @param addr - received IEEE address
 *
 */
void ncp_host_handle_get_local_ieee_addr_response(zb_ret_t status_code, zb_ieee_addr_t addr,
                                                  zb_uint8_t interface_num);

/**
 * Handles Set Local IEEE Address response.
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_set_local_ieee_addr_response(zb_ret_t status_code);


/**
 * Handles Set RxOnWhenIdle PIB attribute response.
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_set_rx_on_when_idle_response(zb_ret_t status_code);


/**
 * Handles Get RxOnWhenIdle PIB attribute response.
 *
 * @param status_code - status of operation
 * @param rx_on_when_idle - RxOnWhenIdle PIB attribute value
 */
void ncp_host_handle_get_rx_on_when_idle_response(zb_ret_t status_code, zb_bool_t rx_on_when_idle);


/**
 * Handles Set Keepalive Timeout response.
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_set_keepalive_timeout_response(zb_ret_t status_code);


/**
 * Handles Get Keepalive Timeout response.
 *
 * @param status_code - status of operation
 * @param keepalive_timeout - Keepalive Timeout value
 */
void ncp_host_handle_get_keepalive_timeout_response(zb_ret_t status_code, zb_uint32_t keepalive_timeout);


/**
 * Handles Set End Device Timeout response.
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_set_end_device_timeout_response(zb_ret_t status_code);


/**
 * Handles Get End Device Timeout response.
 *
 * @param status_code - status of operation
 * @param timeout - End Device Timeout value
 */
void ncp_host_handle_get_end_device_timeout_response(zb_ret_t status_code, zb_uint8_t timeout);



/**
 * Handles Set Zigbee Channel Mask Response
 *
 * @param status_code - status of operation
 *
 */
void ncp_host_handle_set_zigbee_channel_mask_response(zb_ret_t status_code);

/**
 * Handles Get Zigbee Channel Mask Response
 *
 * @param status_code - status of operation
 * @param channel_list_len - count of channel structures
 * @param channel_page - Channel Page
 * @param channel mask - Channel Mask
 *
 */
void ncp_host_handle_get_zigbee_channel_mask_response(zb_ret_t status_code, zb_uint8_t channel_list_len,
                                                      zb_uint8_t channel_page, zb_uint32_t channel_mask);

/**
 * Handles Get Zigbee Current Channel Response
 *
 * @param status_code - status of operation
 * @param page - current page
 * @param channel - current channel
 *
 */
void ncp_host_handle_get_zigbee_channel_response(zb_ret_t status_code, zb_uint8_t page, zb_uint8_t channel);

/**
 * Handles Set Zigbee PAN ID Response
 *
 * @param status_code - status of operation
 *
 */
void ncp_host_handle_set_zigbee_pan_id_response(zb_ret_t status_code);

/**
 * Handles Get Zigbee PAN ID Response
 *
 * @param status_code - status of operation
 * @param pan_id - status of operation
 *
 */
void ncp_host_handle_get_zigbee_pan_id_response(zb_ret_t status_code, zb_uint16_t pan_id);

/**
 * Handles Set Zigbee Role Response
 *
 * @param status_code - status of operation
 *
 */
void ncp_host_handle_set_zigbee_role_response(zb_ret_t status_code);

/**
 * Handles Get Zigbee Role Response
 *
 * @param status_code - status of operation
 * @param zigbee_role - device role in a Zigbee network
 *
 */
void ncp_host_handle_get_zigbee_role_response(zb_ret_t status_code, zb_uint8_t zigbee_role);

/**
 * Handles Set NWK Key Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_set_nwk_key_response(zb_ret_t status_code);

/**
 * Handles Get NWK Keys Response
 *
 * @param status_code - status of operation
 * @param nwk_key1
 * @param key_number1
 * @param nwk_key2
 * @param key_number2
 * @param nwk_key3
 * @param key_number3
 *
 */
void ncp_host_handle_get_nwk_keys_response(zb_ret_t status_code,
                                           zb_uint8_t *nwk_key1, zb_uint8_t key_number1,
                                           zb_uint8_t *nwk_key2, zb_uint8_t key_number2,
                                           zb_uint8_t *nwk_key3, zb_uint8_t key_number3);

/**
 * Handles Get APS Key by IEEE Response
 *
 * @param status_code - status of operation
 * @param aps_key
 */
void ncp_host_handle_get_aps_key_by_ieee_response(zb_ret_t status_code, const zb_uint8_t *aps_key);


/**
 * Handles Get Parent Address Response
 *
 * @param status code - status of operation
 * @param parent_address
 */
void ncp_host_handle_get_parent_address_response(zb_ret_t status_code, zb_uint16_t parent_address);

/**
 * Handles Get Extended PAN ID Response
 *
 * @param status code - status of operation
 * @param ext_pan_id - extended PAN ID
 */
void ncp_host_handle_get_extended_pan_id_response(zb_ret_t status_code,
                                                  const zb_ieee_addr_t ext_pan_id);

/**
 * Handles Get Coordinator Version Response
 *
 * @param status code - status of operation
 * @param version - coordinator version
 */
void ncp_host_handle_get_coordinator_version_response(zb_ret_t status_code, zb_uint8_t version);

/**
 * Handles Get Short Address Response
 *
 * @param status code - status of operation
 * @param address - current short address of the device
 */
void ncp_host_handle_get_short_address_response(zb_ret_t status_code, zb_uint16_t address);

/**
 * Handles Get Trust Center Address Response
 *
 * @param status code - status of operation
 * @param address - current Trust Center IEEE address
 */
void ncp_host_handle_get_trust_center_address_response(zb_ret_t status_code,
                                                       const zb_ieee_addr_t address);

/* NWK API */

/**
 * Handles NWK Get IEEE By Short Response
 *
 * @param status_code - status of operation
 * @param ieee_addr - IEEE address
 */
void ncp_host_handle_nwk_get_ieee_by_short_response(zb_ret_t status_code, zb_ieee_addr_t ieee_addr);

/* TODO: Implement returning of the neighobr table entity*/
/**
 * Handles NWK Get Neighbor By IEEE Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_get_neighbor_by_ieee_response(zb_ret_t status_code);

/**
 * Handles NWK Get Short By IEEE Response
 *
 * @param status_code - status of operation
 * @param short_addr - short address from the local address translation table
 */
void ncp_host_handle_nwk_get_short_by_ieee_response(zb_ret_t status_code, zb_uint16_t short_addr);

#ifdef ZB_APSDE_REQ_ROUTING_FEATURES
void ncp_host_handle_nwk_route_reply_indication_adapter(zb_bufid_t buf);
void ncp_host_handle_nwk_route_request_send_indication_adapter(zb_bufid_t buf);
void ncp_host_handle_nwk_route_record_send_indication_adapter(zb_bufid_t buf);
#endif
/**
 * Handles Set Fast Poll Interval PIM Attribute Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_set_fast_poll_interval_response(zb_ret_t status_code);


/**
 * Handles Set Long Poll Interval PIM Attribute Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_set_long_poll_interval_response(zb_ret_t status_code);


/**
 * Handles Start Fast Poll Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_start_fast_poll_response(zb_ret_t status_code);


/**
 * Handles Start Poll Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_start_poll_response(zb_ret_t status_code);


/**
 * Handles Stop Fast Poll Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_stop_fast_poll_response(zb_ret_t status_code, zb_uint8_t ncp_tsn,
  zb_stop_fast_poll_result_t stop_result);


/**
 * Handles Stop Poll Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_stop_poll_response(zb_ret_t status_code);


/**
 * Handles Enable turbo poll Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_enable_turbo_poll_response(zb_ret_t status_code);


/**
 * Handles Disable turbo poll Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_disable_turbo_poll_response(zb_ret_t status_code);


/**
 * Handles Start Turbo Poll Packets Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_start_turbo_poll_packets_response(zb_ret_t status_code);


/**
 * Handles Start Turbo Poll Continuous Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_start_turbo_poll_continuous_response(zb_ret_t status_code);


/**
 * Handles Permit Turbo Poll Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_permit_turbo_poll_response(zb_ret_t status_code);


/**
 * Handles Set Keepalive Mode Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_set_keepalive_mode_response(zb_ret_t status_code);


/**
 * Handles Turbo Poll Continuous Leave Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_turbo_poll_continuous_leave_response(zb_ret_t status_code);


/**
 * Handles Turbo Poll Packets Leave Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_turbo_poll_packets_leave_response(zb_ret_t status_code);


/**
 * Handles Set Fast Poll Timeout Mode Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_nwk_set_fast_poll_timeout_response(zb_ret_t status_code);


/**
 * Handles Get Long Poll Interval Response
 *
 * @param status - status of operation
 * @param ncp_tsn - NCP TSN value
 * @param interval - fast poll interval value
 */
void ncp_host_handle_nwk_get_long_poll_interval_response(zb_ret_t status, zb_uint8_t ncp_tsn, zb_uint32_t interval);


/**
 * Handles Get "In Fast Poll" Flag Response
 *
 * @param status - status of operation
 * @param ncp_tsn - NCP TSN value
 * @param in_fast_poll_status - "In Fast Poll" flag value
 */
void ncp_host_handle_nwk_get_in_fast_poll_flag_response(zb_ret_t status, zb_uint8_t ncp_tsn, zb_uint8_t in_fast_poll_status);


/* Security API */
/**
 * Handles NWK Secur Add Remote Device Installcode Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_secur_add_ic_response(zb_ret_t status_code);

/* AF API */
/**
 * Handles AF Set Simple Descriptor Response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_af_set_simple_descriptor_response(zb_ret_t status_code);

/**
 * Handles AF Delete Endpoint command response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_af_delete_endpoint_response(zb_ret_t status_code);

/**
 * Handles AF Set Node Descriptor command response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_af_set_node_descriptor_response(zb_ret_t status_code);

/**
 * Handles AF Set Power Descriptor command response
 *
 * @param status_code - status of operation
 */
void ncp_host_handle_af_set_power_descriptor_response(zb_ret_t status_code);


#endif /* NCP_HOST_HL_TRANSPORT_INTERNAL_API_H */
