/*
 * ZBOSS Zigbee software protocol stack
 *
 * Copyright (c) 2012-2020 DSR Corporation, Denver CO, USA.
 * http://www.dsr-zboss.com
 * http://www.dsr-corporation.com
 * All rights reserved.
 *
 *
 * Use in source and binary forms, redistribution in binary form only, with
 * or without modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 2. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 3. This software must only be used in or with a processor manufactured by Nordic
 *    Semiconductor ASA, or in or with a processor manufactured by a third party that
 *    is used in combination with a processor manufactured by Nordic Semiconductor.
 *
 * 4. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* PURPOSE: ZCL Diagnostics cluster specific commands handling
*/

#define ZB_TRACE_FILE_ID 2065
#include "zb_common.h"
#include "zb_time.h"
#include "zb_zdo.h"
#include "zb_zcl.h"
#include "zcl/zb_zcl_diagnostics.h"

#if defined(ZB_ZCL_SUPPORT_CLUSTER_DIAGNOSTICS)

zb_zcl_diagnostics_data_t diagnostics_data;

/** @internal
    @{
*/

zb_ret_t check_value_diagnostics_server(zb_uint16_t attr_id, zb_uint8_t endpoint, zb_uint8_t *value);

void zb_zcl_diagnostics_init_server()
{
  zb_zcl_add_cluster_handlers(ZB_ZCL_CLUSTER_ID_DIAGNOSTICS,
                              ZB_ZCL_CLUSTER_SERVER_ROLE,
                              check_value_diagnostics_server,
                              (zb_zcl_cluster_write_attr_hook_t)NULL,
                              (zb_zcl_cluster_handler_t)NULL);
}

void zb_zcl_diagnostics_init_client()
{
  zb_zcl_add_cluster_handlers(ZB_ZCL_CLUSTER_ID_DIAGNOSTICS,
                              ZB_ZCL_CLUSTER_CLIENT_ROLE,
                              (zb_zcl_cluster_check_value_t)NULL,
                              (zb_zcl_cluster_write_attr_hook_t)NULL,
                              (zb_zcl_cluster_handler_t)NULL);
}

zb_ret_t check_value_diagnostics_server(zb_uint16_t attr_id, zb_uint8_t endpoint, zb_uint8_t *value)
{
  ZVUNUSED(attr_id);
  ZVUNUSED(value);
  ZVUNUSED(endpoint);

  /* All values for mandatory attributes are allowed, extra check for
   * optional attributes is needed */

  return RET_OK;
}
/**
 *  @} internal
*/
static void update_average_max_per_aps()
{
  zb_uint16_t all_aps = diagnostics_data.apsTxBcast + diagnostics_data.apsTxUcastSuccess
      +diagnostics_data.apsTxUcastFail;
  diagnostics_data.averageMACRetryPerAPSMessageSent = (all_aps) ?
      (diagnostics_data.macTxBcast+diagnostics_data.macTxUcast) / all_aps : 1;
}

void zb_zcl_diagnostics_inc(zb_uint16_t attr_id, zb_uint8_t value)
{
  switch(attr_id)
  {

  case ZB_ZCL_ATTR_DIAGNOSTICS_NUMBER_OF_RESETS_ID:
    ++(diagnostics_data.numberOfResets);
    break;
  case ZB_ZCL_ATTR_DIAGNOSTICS_PACKET_BUFFER_ALLOCATE_FAILURES_ID:
    ++(diagnostics_data.packetBufferAllocateFeatures);
    break;
  case ZB_ZCL_ATTR_DIAGNOSTICS_JOIN_INDICATION_ID:
    ++(diagnostics_data.joinIndication);
    break;
  case ZB_ZCL_ATTR_DIAGNOSTICS_MAC_RX_BCAST_ID:
    ++(diagnostics_data.macRxBcast);
    break;
  case ZB_ZCL_ATTR_DIAGNOSTICS_MAC_TX_BCAST_ID:
    ++(diagnostics_data.macTxBcast);
    update_average_max_per_aps();
    break;
  case ZB_ZCL_ATTR_DIAGNOSTICS_MAC_TX_UCAST_ID:
    ++(diagnostics_data.macTxUcast);
    update_average_max_per_aps();
    break;
  case ZB_ZCL_ATTR_DIAGNOSTICS_APS_TX_BCAST_ID:
    ++(diagnostics_data.apsTxBcast);
    update_average_max_per_aps();
    break;
  case ZB_ZCL_ATTR_DIAGNOSTICS_APS_TX_UCAST_SUCCESS_ID:
    ++(diagnostics_data.apsTxUcastSuccess);
    update_average_max_per_aps();
    break;
  case ZB_ZCL_ATTR_DIAGNOSTICS_APS_TX_UCAST_FAIL_ID:
    ++(diagnostics_data.apsTxUcastFail);
    update_average_max_per_aps();
    break;
  case ZB_ZCL_ATTR_DIAGNOSTICS_AVERAGE_MAC_RETRY_PER_APS_ID:
    break;
  case ZB_ZCL_ATTR_DIAGNOSTICS_LAST_LQI_ID:
    diagnostics_data.lastMessageLQI = value;
    break;
  case ZB_ZCL_ATTR_DIAGNOSTICS_LAST_RSSI_ID:
    diagnostics_data.lastMessageRSSI = value;
    break;
  default:
    break;
  }
}

#endif /* defined(ZB_ZCL_SUPPORT_CLUSTER_DIAGNOSTICS) */
