/*
 * ZBOSS Zigbee software protocol stack
 *
 * Copyright (c) 2012-2020 DSR Corporation, Denver CO, USA.
 * http://www.dsr-zboss.com
 * http://www.dsr-corporation.com
 * All rights reserved.
 *
 *
 * Use in source and binary forms, redistribution in binary form only, with
 * or without modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 2. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 3. This software must only be used in or with a processor manufactured by Nordic
 *    Semiconductor ASA, or in or with a processor manufactured by a third party that
 *    is used in combination with a processor manufactured by Nordic Semiconductor.
 *
 * 4. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*  PURPOSE: ZBOSS core API
*/

#define ZB_TRACE_FILE_ID 61
#include "zb_common.h"
#include "zb_bdb_internal.h"
#include "zb_scheduler.h"
#include "zb_nwk.h"
#include "zboss_api.h"
#include "zb_zdo.h"
#include "zb_aps.h"
#include "zb_bdb_internal.h"


#ifdef ZB_BDB_MODE
/*! \addtogroup zboss_bdb_api */
/*! @{ */

void zb_set_bdb_primary_channel_set(zb_uint32_t channel_mask)
{
  ZB_BDB().bdb_primary_channel_set = channel_mask;
}

zb_uint32_t zb_get_bdb_primary_channel_set()
{
  return ZB_BDB().bdb_primary_channel_set;
}

void zb_set_bdb_secondary_channel_set(zb_uint32_t channel_mask)
{
  ZB_BDB().bdb_secondary_channel_set = channel_mask;
}

zb_uint32_t zb_get_bdb_secondary_channel_set()
{
  return ZB_BDB().bdb_secondary_channel_set;
}

void zb_bdb_set_legacy_device_support(zb_uint8_t enable)
{
  /* WARNING: Turn off link key exchange if legacy device support (<ZB3.0) is neeeded */
  if ((enable != 0) && (enable != 1))
  {
    ZB_ASSERT(0);
  }
#ifndef ZB_LITE_NO_TRUST_CENTER_REQUIRE_KEY_EXCHANGE
  zb_aib_tcpol_set_update_trust_center_link_keys_required((zb_bool_t)!enable);
#endif
}

void zb_set_bdb_commissioning_mode(zb_uint8_t commissioning_mode)
{
  ZB_BDB().bdb_commissioning_mode = commissioning_mode;
}

/*! @} */
#endif /*#ifdef ZB_BDB_MODE*/

#ifdef ZB_COORDINATOR_ROLE
void zb_set_network_coordinator_role_with_mode(zb_uint32_t channel_mask, zb_uint8_t mode)
{
  zb_set_network_role_with_mode_common(ZB_NWK_DEVICE_TYPE_COORDINATOR,
                                       channel_mask,
                                       (zb_commissioning_type_t)mode);
}

void zb_set_network_coordinator_role(zb_uint32_t channel_mask)
{
  zb_set_network_coordinator_role_with_mode(channel_mask, ZB_COMMISSIONING_BDB);
}

void zb_set_network_coordinator_role_legacy(zb_uint32_t channel_mask)
{
  zb_set_network_coordinator_role_with_mode(channel_mask, ZB_COMMISSIONING_CLASSIC);
}
#endif  /* ZB_COORDINATOR_ROLE */

#ifdef ZB_ROUTER_ROLE
void zb_set_network_router_role_with_mode(zb_uint32_t channel_mask, zb_commissioning_type_t bdb_mode)
{
  zb_set_network_role_with_mode_common(ZB_NWK_DEVICE_TYPE_ROUTER, channel_mask, bdb_mode);
}
#endif

#ifdef ZB_ROUTER_ROLE
void zb_set_network_router_role(zb_uint32_t channel_mask)
{
  zb_set_network_router_role_with_mode(channel_mask, ZB_COMMISSIONING_BDB);
}

void zb_set_network_router_role_legacy(zb_uint32_t channel_mask)
{
  zb_set_network_router_role_with_mode(channel_mask, ZB_COMMISSIONING_CLASSIC);
}
#endif  /* ZB_ROUTER_ROLE */

#ifdef ZB_ED_FUNC
void zb_set_network_ed_role_with_mode(zb_uint32_t channel_mask, zb_commissioning_type_t bdb_mode)
{
  zb_set_network_role_with_mode_common(ZB_NWK_DEVICE_TYPE_ED, channel_mask, bdb_mode);
}

void zb_set_network_ed_role(zb_uint32_t channel_mask)
{
  zb_set_network_ed_role_with_mode(channel_mask, ZB_COMMISSIONING_BDB);
}

void zb_set_network_ed_role_legacy(zb_uint32_t channel_mask)
{
  zb_set_network_ed_role_with_mode(channel_mask, ZB_COMMISSIONING_CLASSIC);
}
#endif  /* ZB_ED_FUNC */

void zb_set_network_coordinator_role_ext(zb_channel_list_t channel_list)
{
  zb_set_network_role_with_mode_common_ext(ZB_NWK_DEVICE_TYPE_COORDINATOR,
                                           channel_list,
                                           ZB_COMMISSIONING_BDB);
}

void zb_set_network_ed_role_ext(zb_channel_list_t channel_list)
{
  zb_set_network_role_with_mode_common_ext(ZB_NWK_DEVICE_TYPE_ED,
                                           channel_list,
                                           ZB_COMMISSIONING_BDB);
}

#ifdef ZB_SE_COMMISSIONING

#ifdef ZB_COORDINATOR_ROLE
void zb_se_set_network_coordinator_role(zb_uint32_t channel_mask)
{
  zb_set_network_coordinator_role_with_mode(channel_mask, ZB_COMMISSIONING_SE);
}
#endif

#ifdef ZB_ED_FUNC
void zb_se_set_network_ed_role(zb_uint32_t channel_mask)
{
  zb_set_network_ed_role_with_mode(channel_mask, ZB_COMMISSIONING_SE);
}
#endif

#ifdef ZB_ROUTER_ROLE
void zb_se_set_network_router_role(zb_uint32_t channel_mask)
{
  /* ZSE_CTXC().commissioning.auto_join = 1; */
  zb_set_network_router_role_with_mode(channel_mask, ZB_COMMISSIONING_SE);
}
#endif

#endif  /*  ZB_SE_COMMISSIONING */


#ifdef ZB_BDB_MODE
zb_bool_t zb_bdb_is_factory_new()
{
  return (zb_bool_t)!bdb_joined();
}
#endif

void zboss_main_loop()
{
  while (!ZB_SCHEDULER_IS_STOP())
  {
    zb_sched_loop_iteration();
  }
}

void zboss_main_loop_iteration()
{
  zb_sched_loop_iteration();
}

zb_nwk_device_type_t zb_get_network_role(void)
{
  return zb_get_device_type();
}
