/*
 * ZBOSS Zigbee software protocol stack
 *
 * Copyright (c) 2012-2020 DSR Corporation, Denver CO, USA.
 * http://www.dsr-zboss.com
 * http://www.dsr-corporation.com
 * All rights reserved.
 *
 *
 * Use in source and binary forms, redistribution in binary form only, with
 * or without modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 2. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 3. This software must only be used in or with a processor manufactured by Nordic
 *    Semiconductor ASA, or in or with a processor manufactured by a third party that
 *    is used in combination with a processor manufactured by Nordic Semiconductor.
 *
 * 4. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* PURPOSE: ZDO common functions, both client and server side
*/

#ifndef ZDO_COMMON_H
#define ZDO_COMMON_H 1

#define ZB_TSN_HASH(tsn) ZB_1INT_HASH_FUNC(tsn) % ZDO_TRAN_TABLE_SIZE

zb_uint8_t zdo_send_req_by_short(zb_uint16_t command_id, zb_uint8_t param, zb_callback_t cb,
                                 zb_uint16_t addr, zb_uint8_t resp_counter);
zb_uint8_t zdo_send_req_by_long(zb_uint8_t command_id, zb_uint8_t param, zb_callback_t cb,
                                zb_ieee_addr_t addr);
void zdo_send_resp_by_short(zb_uint16_t command_id, zb_uint8_t param, zb_uint16_t addr);

/*
   Signal user that response came.
   param param - index of buffer with response
*/
zb_ret_t zdo_af_resp(zb_uint8_t param);

void zdo_handle_mgmt_leave_rsp(zb_uint16_t src);


/*! \addtogroup legacy_callbacks */
/*! @{ */

/**
   Enum ZBOSS Core callback types.
  */
typedef enum zb_zboss_core_callbacks_e
{
  #ifdef ZB_PRO_ADDRESS_ASSIGNMENT_CB
  ZB_DEVICE_ASSOCIATED_CB,          /*!< Informs application that some device has been associated.
                                         Use it on routers and coordinators.
                                         See @ref zb_addr_assignment_cb_t for callback type. */
#endif  /* ZB_PRO_ADDRESS_ASSIGNMENT_CB */
  ZB_DEVICE_ANNCE_CB,               /*!< Informs application that some device has joined to network.
                                         Use this callback on routers and coordinators.
                                         See @ref zb_device_annce_cb_t for callback type. */
  ZB_LEAVE_INDICATION_CB,           /*!< Informs application that some device has left network.
                                         Use it on Routers and coordinators.
                                         See @ref zb_callback_t for callback type. */
  ZB_LEAVE_CB,                      /*!< Called when device leaves itself from network.
                                         See @ref zb_callback_t for callback type. */
  ZB_ASSERT_INDICATION_CB,          /*!< Called when assert was occured.
                                         See @ref zb_callback_t for callback type. */
  ZB_ZCL_DEVICE_CB,                 /*!< Informs application that ZCL command was received.
                                         See @ref zb_callback_t for callback type. */
} zb_zboss_core_callbacks_t;

#define SET_ZBOSS_CB(cb) (void (*)()) cb

/**  Register ZBOSS Core callbacks.
  *
  * Each callback is triggered by it's specific event denoted in cbid parameter.
  * Reference to @ref zb_zboss_core_callbacks_e.
  *
  * @param cbid - callback id used to specify callback type
  * @param cb_ptr - pointer to callback function
  *
  * @b Example:
  * @code
  *   ...
  *   zb_register_zboss_callback(ZB_DEVICE_ANNCE_CB, SET_ZBOSS_CB(dev_annce_cb));
  *   ...
  *
  *   void dev_annce_cb(zb_zdo_device_annce_t *da)
  *   {
  *     /-- user code --/
  *   }
  * @endcode
  */
void zb_register_zboss_callback(zb_zboss_core_callbacks_t cb_id, void (*cb_ptr)());


void zb_zdo_nib_set_attr(
  zb_uint8_t param,
  zb_uint8_t attr,
  void *value,
  zb_ushort_t value_size,
  zb_callback_t cb);

void zdo_comm_set_permit_join(zb_uint8_t param, zb_callback_t cb);

#endif /* ZDO_COMMON_H */
